use bevy::prelude::*;

// --- Resources ---
#[derive(Resource)]
pub struct EnemySpawnTimer {
    pub timer: Timer,
}

#[derive(Resource)]
pub struct PlayerLives {
    pub count: u32,
}

#[derive(Resource)]
pub struct PlayerRespawnTimer {
    pub timer: Timer,
}

// New struct to define formation positions
#[derive(Clone, Debug)]
pub struct FormationLayout {
    pub name: String, // Optional name for debugging/identification
    pub positions: Vec<Vec3>,
}

// Default implementation for easy initialization
impl Default for FormationLayout {
    fn default() -> Self {
        // Default to the original grid formation for now
        let mut positions = Vec::with_capacity(crate::constants::FORMATION_ENEMY_COUNT);
        for i in 0..crate::constants::FORMATION_ENEMY_COUNT {
            let row = i / crate::constants::FORMATION_COLS;
            let col = i % crate::constants::FORMATION_COLS;
            let target_x = (col as f32 - (crate::constants::FORMATION_COLS as f32 - 1.0) / 2.0) * crate::constants::FORMATION_X_SPACING;
            let target_y = crate::constants::FORMATION_BASE_Y - (row as f32 * crate::constants::FORMATION_Y_SPACING);
            positions.push(Vec3::new(target_x, target_y, 0.0));
        }
        FormationLayout {
            name: "Default Grid".to_string(),
            positions,
        }
    }
}

// Configuration for a single stage
#[derive(Clone, Debug)]
pub struct StageConfig {
    pub formation_layout: FormationLayout,
    pub enemy_count: usize, // Allow overriding enemy count per stage
    pub attack_patterns: Vec<crate::components::AttackPattern>, // Possible patterns for this stage
    pub attack_dive_interval: f32, // Time between attack dives for this stage
    pub enemy_speed_multiplier: f32, // Speed multiplier for this stage
    pub enemy_shoot_interval: f32, // Shoot interval for this stage
}

// Resource to hold all stage configurations
#[derive(Resource, Debug, Clone)]
pub struct StageConfigurations {
    pub stages: Vec<StageConfig>,
}

impl Default for StageConfigurations {
    fn default() -> Self {
        use crate::components::AttackPattern;
        use crate::constants::*; // Import constants for default values

        // Define configurations for a few stages
        let stage1 = StageConfig {
            formation_layout: FormationLayout::default(), // Use the default grid
            enemy_count: FORMATION_ENEMY_COUNT,
            attack_patterns: vec![AttackPattern::SwoopDive],
            attack_dive_interval: 3.0,
            enemy_speed_multiplier: 1.0,
            enemy_shoot_interval: ENEMY_SHOOT_INTERVAL,
        };

        let stage2_layout = {
            let mut positions = Vec::new();
            let radius = WINDOW_WIDTH / 4.0;
            let center_y = FORMATION_BASE_Y - 50.0;
            let count = 16; // Example: Fewer enemies in a circle
            for i in 0..count {
                let angle = (i as f32 / count as f32) * 2.0 * std::f32::consts::PI;
                positions.push(Vec3::new(angle.cos() * radius, center_y + angle.sin() * radius, 0.0));
            }
            FormationLayout { name: "Circle".to_string(), positions }
        };
        let stage2 = StageConfig {
            formation_layout: stage2_layout,
            enemy_count: 16,
            attack_patterns: vec![AttackPattern::SwoopDive, AttackPattern::DirectDive], // Add direct dive
            attack_dive_interval: 2.5, // Faster dives
            enemy_speed_multiplier: 1.2, // Faster enemies
            enemy_shoot_interval: ENEMY_SHOOT_INTERVAL * 0.8, // Faster shooting
        };

        // Add more stages here...

        StageConfigurations {
            stages: vec![stage1, stage2], // Add more stages as needed
        }
    }
}
#[derive(Resource)]
pub struct Score {
    pub value: u32,
}

#[derive(Resource)]
pub struct CurrentStage {
    pub number: u32,
    pub waiting_for_clear: bool, // Flag to check if we should check for stage clear
}

#[derive(Resource)]
pub struct FormationState {
    pub next_slot_index: usize,
    pub total_spawned_this_stage: usize,
    pub formation_complete: bool, // Flag to indicate if all enemies are in position
}

#[derive(Resource)]
pub struct AttackDiveTimer {
    pub timer: Timer,
}